<?php

/**
 *
 * @package    EasyAds
 * @author     Abude Bayassi <abude.bayassi@easyads.io>
 * @link       https://www.easyads.io
 * @copyright  2017 EasyAds (https://www.easyads.io)
 * @license    https://www.easyads.io
 * @since      1.0
 */

namespace app\modules\admin\controllers;

use app\models\ListingStat;
use app\models\Listing;
use app\models\ListingSearch;
use yii\web\NotFoundHttpException;

/**
 * Controls the actions for ads section
 *
 * @Class AdsController
 * @package app\modules\admin\controllers
 */
class ListingsController extends \app\modules\admin\yii\web\Controller
{

    /**
     * Lists all Ads
     *
     * @return string
     */
    public function actionIndex()
    {
        $searchModel = new ListingSearch();
        $dataProvider = $searchModel->search(request()->queryParams);
        $dataProvider->pagination->pageSize=10;
        return $this->render('list', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays stats for a specific ad
     *
     * @param $id
     * @return string
     */
    public function actionStats($id)
    {
        return $this->render('stats', [
            'model' => $this->findStat($id),
            'ad' => $this->findModel($id),
        ]);
    }

    /**
     * Lists all Ads with pending status
     *
     * @return string
     */
    public function actionPendings()
    {
        $searchModel = new ListingSearch();
        $dataProvider = $searchModel->search(request()->queryParams);
        $dataProvider->query->andWhere(['status' => Listing::STATUS_PENDING_APPROVAL]);
        $dataProvider->pagination->pageSize=10;
        return $this->render('list-pendings', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Handles the activation for a specific ad
     *
     * @param $id
     * @return void|\yii\web\Response
     */
    public function actionActivate($id)
    {
        $model = $this->findModel($id);
        Listing::sendAdActivateEmail($model);
        $model->status = Listing::STATUS_ACTIVE;
        if (!$model->save(false)) {
            notify()->addError(t('app','Couldn\'t complete your action, please try again later.'));
            return;
        }
        notify()->addSuccess(t('app','Your action is complete'));
        return $this->redirect(request()->referrer);
    }

    /**
     * Handles the deactivation for a specific ad
     *
     * @param $id
     * @return void|\yii\web\Response
     */
    public function actionDeactivate($id)
    {
        $model = $this->findModel($id);
        Listing::sendAdDeactivateEmail($model);
        $model->status = Listing::STATUS_DEACTIVATED;
        if (!$model->save(false)) {
            notify()->addError(t('app','Couldn\'t complete your action, please try again later.'));
            return;
        }
        notify()->addSuccess(t('app','Your action is complete'));
        return $this->redirect(request()->referrer);
    }

    /**
     * View a specific ad in frontend
     *
     * @param $id
     * @return \yii\web\Response
     */
    public function actionAdminPreviewAd($id)
    {
        $ad = Listing::findOne(['listing_id' => (int)$id]);
        if (empty($ad)) {
            notify()->addWarning(t('app', 'Please specify a valid ad'));
            return $this->redirect(['/admin/listings']);
        }

        app()->customer->switchIdentity($ad->customer);

        session()->set('impersonating_customer', true);
        if ($ad->status == Listing::STATUS_PENDING_APPROVAL) {
            session()->set('impersonating_customer_back_url', url(['/admin/listings/pendings']));
        }
        session()->set('impersonating_customer_back_url', url(['/admin/listings']));
        return $this->redirect(['/listing/preview/' . $ad->slug]);
    }

    /**
     * @param $id
     * @return static
     * @throws NotFoundHttpException
     */
    protected function findModel($id)
    {
        if (($model = Listing::findOne($id)) !== null) {
            return $model;
        }
        throw new NotFoundHttpException('The requested page does not exist.');
    }

    /**
     * @param $id
     * @return static
     * @throws NotFoundHttpException
     */
    protected function findStat($id)
    {
        if (($model = ListingStat::findOne($id)) !== null) {
            return $model;
        }
        return false;
    }
}
